<?php
/**
 * Users List Template
 *
 * @package WooCommerce Wallet Management
 * @version 1.0.0
 */

namespace DDWCWalletManagement\Templates\Admin\Users;

use DDWCWalletManagement\Helper\Users\DDWCWM_Users_Helper;

defined( 'ABSPATH' ) || exit();

if ( ! class_exists( 'DDWCWM_Users_List_Template' ) ) {

	/**
	 * Users list class
	 */
	class DDWCWM_Users_List_Template extends \WP_List_table {
		/**
		 * Users Helper Variable
		 *
		 * @var object
		 */
		protected $user_helper;

		/**
		 * Class constructor
		 */
		public function __construct() {
            $this->user_helper = new DDWCWM_Users_Helper();

			parent::__construct( [
				'singular' => esc_html__( 'User List', 'ddwc-wallet-management' ),
				'plural'   => esc_html__( 'Users List', 'ddwc-wallet-management' ),
				'ajax'     => false,
			] );
		}

		/**
		 * Prepare Items
		 *
		 * @return void
		 */
		public function prepare_items() {
			$this->_column_headers = $this->get_column_info();

			$request_scheme = is_ssl() ? 'https' : 'http';
			$current_url    = "$request_scheme://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";

            if ( strpos( $current_url, '_wp_http_referer' ) !== false ) {
                $new_url = remove_query_arg( [ '_wp_http_referer', '_wpnonce' ], stripslashes( $current_url ) );
				wp_safe_redirect( $new_url );
				exit();
            }

            $search = ! empty( $_GET[ 's' ] ) ? sanitize_text_field( wp_unslash( $_GET[ 's' ] ) ) : '';

			$per_page     = $this->get_items_per_page( 'users_per_page', 20 );
			$current_page = $this->get_pagenum();
			$total_items  = self::ddwcwm_users_count( $search );

			$this->set_pagination_args( [
				'total_items' => $total_items,
				'per_page'    => $per_page,
			] );

			$data = self::ddwcwm_get_users( $per_page, $current_page, $search );

			// usort( $data, [ $this, 'usort_reorder' ] );

			$this->items = $data;
		}

		/**
		 * Usort
		 *
		 * @param int $first First value.
		 * @param int $second Second value.
		 * @return $result
		 */
		public function usort_reorder( $first, $second ) {
			$orderby = ! empty( $_GET[ 'orderby' ] ) ? $_GET[ 'orderby' ] : 'id';
			$order   = ! empty( $_GET[ 'order' ] ) ? $_GET[ 'order' ] : 'asc';

			$result  = strnatcmp( $first[ $orderby ], $second[ $orderby ] );

			return 'asc' === $order ? $result : -$result;
		}

		/**
		 * Record count
		 *
		 * @param string $search Search.
		 * @return $count
		 */
		public function ddwcwm_users_count( $search = '' ) {
			$args = [
				'search'         => '*' . esc_attr( $search ) . '*',
				'order'          => ! empty( $_GET[ 'order' ] ) ? $_GET[ 'order' ] : 'DESC',
				'orderby'        => ! empty( $_GET[ 'orderby' ] ) ? $_GET[ 'orderby' ] : 'ID',
				'search_columns' => [ 'user_nicename', 'ID', 'user_login', 'user_email' ],
			];

			if ( ! empty( $args[ 'orderby' ] ) && 'balance' === $args[ 'orderby' ] ) {
				$args[ 'meta_key' ] = '_ddwcwm_wallet_balance';
				$args[ 'orderby' ]  = 'meta_value_num';
            }

			$user_query = new \WP_User_Query( $args );

            return $user_query->get_total();
		}

		/**
		 * Fetch Users
		 *
		 * @param int $per_page Per Page.
		 * @param int $current_page Page.
		 * @param string $search Search.
		 * @return array $users
		 */
		public function ddwcwm_get_users( $per_page, $current_page = 1, $search = '' ) {

            $data = [];

            $off = ( $current_page - 1 ) * $per_page;

			$args = [
				'number'         => $per_page,
				'offset'         => $off,
				'order'          => ! empty( $_GET[ 'order' ] ) ? $_GET[ 'order' ] : 'DESC',
				'orderby'        => ! empty( $_GET[ 'orderby' ] ) ? $_GET[ 'orderby' ] : 'ID',
				'search'         => '*' . esc_attr( $search ) . '*',
				'search_columns' => [ 'user_nicename', 'ID', 'user_login', 'user_email' ],
				'fields'         => [ 'ID', 'user_login', 'user_email' ],
			];

			if ( ! empty( $args[ 'orderby' ] ) && 'balance' === $args[ 'orderby' ] ) {
				$args[ 'meta_key' ] = '_ddwcwm_wallet_balance';
				$args[ 'orderby' ]  = 'meta_value_num';
            }

			$query = new \WP_User_Query( $args );

            $users = $query->get_results();

            if ( ! empty( $users ) ) {
                foreach ( $users as $user ) {
					$user_id             = $user->ID;
					$balance             = $this->user_helper->ddwcwm_get_user_wallet_balance( $user_id );
					$referred_by_user_id = get_user_meta( $user_id, '_ddwcwm_referred_by', true );
					$referred_by         = esc_html__( 'N/A', 'ddwc-wallet-management' );
					if ( ! empty( $referred_by_user_id ) ) {
						$referred_by_user = get_userdata( $referred_by_user_id );
						$referred_by      = "{$referred_by_user->user_login} <{$referred_by_user->user_email}>";
					}

					ob_start();
					?>
					<a href="<?php echo esc_url( admin_url( 'user-edit.php?user_id=' . $user_id ) ); ?>" class="button ddwcwm-tips edit" title="<?php esc_attr_e( 'Edit', 'ddwc-wallet-management' ); ?>"><?php esc_html_e( 'Edit', 'ddwc-wallet-management' ); ?></a>
					<a href="<?php echo esc_url( admin_url( 'edit.php?post_type=shop_order&_customer_user=' . $user_id ) ); ?>" class="button ddwcwm-tips view" title="<?php esc_attr_e( 'View Orders', 'ddwc-wallet-management' ); ?>"><?php esc_html_e( 'View Orders', 'ddwc-wallet-management' ); ?></a>
					<?php
					$actions = ob_get_contents();
					ob_end_clean();

                    $data[] = [
                        'id'          => $user_id,
                        'username'    => $user->user_login,
                        'email'       => $user->user_email,
                        'balance'     => wc_price( $balance ),
                        'referred_by' => $referred_by,
                        'actions'     => $actions
					];
                }
            }

			return apply_filters( 'ddwcwm_users_list_data', $data );
		}

		/**
		 *  No items
		 *
		 * @return void
		 */
		public function no_items() {
			esc_html_e( 'No users avaliable.', 'ddwc-wallet-management' );
		}

		/**
		 * Hidden Columns
		 *
		 * @return array
		 */
		public function get_hidden_columns() {
			return [];
		}

		/**
		 *  Associative array of columns
		 *
		 * @return array
		 */
		public function get_columns() {
			$columns = [
				'id'          => esc_html__( 'ID', 'ddwc-wallet-management' ),
				'username'    => esc_html__( 'Username', 'ddwc-wallet-management' ),
				'email'       => esc_html__( 'Email', 'ddwc-wallet-management' ),
				'balance'     => esc_html__( 'Balance', 'ddwc-wallet-management' ),
				'referred_by' => esc_html__( 'Referred By', 'ddwc-wallet-management' ),
				'actions'     => esc_html__( 'Actions', 'ddwc-wallet-management' ),
			];

			return apply_filters( 'ddwcwm_users_list_columns', $columns );
		}

		/**
		 * Render a column when no column specific method exists.
		 *
		 * @param array  $item Items.
		 * @param string $column_name Name.
		 *
		 * @return mixed
		 */
		public function column_default( $item, $column_name ) {
			if ( array_key_exists( $column_name, $item ) ) {
				return $item[ $column_name ];
			}

			return print_r( $item, true );
		}

		/**
		 * Columns to make sortable.
		 *
		 * @return array
		 */
		public function get_sortable_columns() {
			$sortable_columns = [
				'id'          => [ 'id', true ],
				'username'    => [ 'username', true ],
				'email'       => [ 'email', true ],
				'balance'     => [ 'balance', true ],
				'referred_by' => [ 'referred_by', true ],
			];

			return apply_filters( 'ddwcwm_users_list_sortable_columns', $sortable_columns );
		}

	}

}
